@@ -1,7 +1,7 @@
 /*	$OpenBSD: strnlen.c,v 1.5 2014/06/10 04:17:37 deraadt Exp $	*/
 
 /*
- * Copyright (c) 2010 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2010 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -30,7 +30,7 @@ sudo_strnlen(const char *str, size_t maxlen)
 	const char *cp;
 
 	for (cp = str; maxlen != 0 && *cp != '\0'; cp++, maxlen--)
-		;
+		continue;
 
 	return (size_t)(cp - str);
 }
