@@ -27,7 +27,6 @@
 #include <config.h>
 
 #include <sys/types.h>
-#include <sys/time.h>
 #include <stdio.h>
 #include <stdlib.h>
 #ifdef HAVE_STRING_H
@@ -41,9 +40,7 @@
 #elif defined(HAVE_INTTYPES_H)
 # include <inttypes.h>
 #endif
-#ifdef TIME_WITH_SYS_TIME
-# include <time.h>
-#endif
+#include <time.h>
 #include <limits.h>
 #include <ctype.h>
 
@@ -110,7 +107,7 @@ static int	yyerror(const char *s);
 static int	yylex(void);
        int	yyparse(void);
 
-#line 103 "getdate.y"
+#line 100 "getdate.y"
 #ifndef YYSTYPE_DEFINED
 #define YYSTYPE_DEFINED
 typedef union {
@@ -118,7 +115,7 @@ typedef union {
     enum _MERIDIAN	Meridian;
 } YYSTYPE;
 #endif /* YYSTYPE_DEFINED */
-#line 121 "getdate.c"
+#line 118 "getdate.c"
 #define tAGO 257
 #define tDAY 258
 #define tDAYZONE 259
@@ -386,7 +383,8 @@ short *yyss;
 short *yysslim;
 YYSTYPE *yyvs;
 unsigned int yystacksize;
-#line 322 "getdate.y"
+int yyparse(void);
+#line 319 "getdate.y"
 
 /* Month and day table. */
 static TABLE const MonthDayTable[] = {
@@ -1008,7 +1006,7 @@ main(int argc, char *argv[])
     /* NOTREACHED */
 }
 #endif	/* TEST */
-#line 959 "getdate.c"
+#line 957 "getdate.c"
 /* allocate initial stack or double stack size, up to YYMAXDEPTH */
 #if defined(__cplusplus) || defined(__STDC__)
 static int yygrowstack(void)
@@ -1217,37 +1215,37 @@ yyreduce:
     switch (yyn)
     {
 case 3:
-#line 121 "getdate.y"
+#line 118 "getdate.y"
 {
 	    yyHaveTime++;
 	}
 break;
 case 4:
-#line 124 "getdate.y"
+#line 121 "getdate.y"
 {
 	    yyHaveZone++;
 	}
 break;
 case 5:
-#line 127 "getdate.y"
+#line 124 "getdate.y"
 {
 	    yyHaveDate++;
 	}
 break;
 case 6:
-#line 130 "getdate.y"
+#line 127 "getdate.y"
 {
 	    yyHaveDay++;
 	}
 break;
 case 7:
-#line 133 "getdate.y"
+#line 130 "getdate.y"
 {
 	    yyHaveRel++;
 	}
 break;
 case 9:
-#line 139 "getdate.y"
+#line 136 "getdate.y"
 {
 	    yyHour = yyvsp[-1].Number;
 	    yyMinutes = 0;
@@ -1256,7 +1254,7 @@ case 9:
 	}
 break;
 case 10:
-#line 145 "getdate.y"
+#line 142 "getdate.y"
 {
 	    yyHour = yyvsp[-3].Number;
 	    yyMinutes = yyvsp[-1].Number;
@@ -1265,7 +1263,7 @@ case 10:
 	}
 break;
 case 11:
-#line 151 "getdate.y"
+#line 148 "getdate.y"
 {
 	    yyHour = yyvsp[-3].Number;
 	    yyMinutes = yyvsp[-1].Number;
@@ -1275,7 +1273,7 @@ case 11:
 	}
 break;
 case 12:
-#line 158 "getdate.y"
+#line 155 "getdate.y"
 {
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
@@ -1284,7 +1282,7 @@ case 12:
 	}
 break;
 case 13:
-#line 164 "getdate.y"
+#line 161 "getdate.y"
 {
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
@@ -1295,56 +1293,56 @@ case 13:
 	}
 break;
 case 14:
-#line 174 "getdate.y"
+#line 171 "getdate.y"
 {
 	    yyTimezone = yyvsp[0].Number;
 	    yyDSTmode = DSToff;
 	}
 break;
 case 15:
-#line 178 "getdate.y"
+#line 175 "getdate.y"
 {
 	    yyTimezone = yyvsp[0].Number;
 	    yyDSTmode = DSTon;
 	}
 break;
 case 16:
-#line 183 "getdate.y"
+#line 180 "getdate.y"
 {
 	    yyTimezone = yyvsp[-1].Number;
 	    yyDSTmode = DSTon;
 	}
 break;
 case 17:
-#line 189 "getdate.y"
+#line 186 "getdate.y"
 {
 	    yyDayOrdinal = 1;
 	    yyDayNumber = yyvsp[0].Number;
 	}
 break;
 case 18:
-#line 193 "getdate.y"
+#line 190 "getdate.y"
 {
 	    yyDayOrdinal = 1;
 	    yyDayNumber = yyvsp[-1].Number;
 	}
 break;
 case 19:
-#line 197 "getdate.y"
+#line 194 "getdate.y"
 {
 	    yyDayOrdinal = yyvsp[-1].Number;
 	    yyDayNumber = yyvsp[0].Number;
 	}
 break;
 case 20:
-#line 203 "getdate.y"
+#line 200 "getdate.y"
 {
 	    yyMonth = yyvsp[-2].Number;
 	    yyDay = yyvsp[0].Number;
 	}
 break;
 case 21:
-#line 207 "getdate.y"
+#line 204 "getdate.y"
 {
 	    if (yyvsp[-4].Number >= 100) {
 		yyYear = yyvsp[-4].Number;
@@ -1358,7 +1356,7 @@ case 21:
 	}
 break;
 case 22:
-#line 218 "getdate.y"
+#line 215 "getdate.y"
 {
 	    /* ISO 8601 format.  yyyy-mm-dd.  */
 	    yyYear = yyvsp[-2].Number;
@@ -1367,7 +1365,7 @@ case 22:
 	}
 break;
 case 23:
-#line 224 "getdate.y"
+#line 221 "getdate.y"
 {
 	    /* e.g. 17-JUN-1992.  */
 	    yyDay = yyvsp[-2].Number;
@@ -1376,14 +1374,14 @@ case 23:
 	}
 break;
 case 24:
-#line 230 "getdate.y"
+#line 227 "getdate.y"
 {
 	    yyMonth = yyvsp[-1].Number;
 	    yyDay = yyvsp[0].Number;
 	}
 break;
 case 25:
-#line 234 "getdate.y"
+#line 231 "getdate.y"
 {
 	    yyMonth = yyvsp[-3].Number;
 	    yyDay = yyvsp[-2].Number;
@@ -1391,14 +1389,14 @@ case 25:
 	}
 break;
 case 26:
-#line 239 "getdate.y"
+#line 236 "getdate.y"
 {
 	    yyMonth = yyvsp[0].Number;
 	    yyDay = yyvsp[-1].Number;
 	}
 break;
 case 27:
-#line 243 "getdate.y"
+#line 240 "getdate.y"
 {
 	    yyMonth = yyvsp[-1].Number;
 	    yyDay = yyvsp[-2].Number;
@@ -1406,68 +1404,68 @@ case 27:
 	}
 break;
 case 28:
-#line 250 "getdate.y"
+#line 247 "getdate.y"
 {
 	    yyRelSeconds = -yyRelSeconds;
 	    yyRelMonth = -yyRelMonth;
 	}
 break;
 case 30:
-#line 257 "getdate.y"
+#line 254 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number * 60L;
 	}
 break;
 case 31:
-#line 260 "getdate.y"
+#line 257 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number * 60L;
 	}
 break;
 case 32:
-#line 263 "getdate.y"
+#line 260 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[0].Number * 60L;
 	}
 break;
 case 33:
-#line 266 "getdate.y"
+#line 263 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number;
 	}
 break;
 case 34:
-#line 269 "getdate.y"
+#line 266 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number;
 	}
 break;
 case 35:
-#line 272 "getdate.y"
+#line 269 "getdate.y"
 {
 	    yyRelSeconds++;
 	}
 break;
 case 36:
-#line 275 "getdate.y"
+#line 272 "getdate.y"
 {
 	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 37:
-#line 278 "getdate.y"
+#line 275 "getdate.y"
 {
 	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	}
 break;
 case 38:
-#line 281 "getdate.y"
+#line 278 "getdate.y"
 {
 	    yyRelMonth += yyvsp[0].Number;
 	}
 break;
 case 39:
-#line 286 "getdate.y"
+#line 283 "getdate.y"
 {
 	    if (yyHaveTime && yyHaveDate && !yyHaveRel)
 		yyYear = yyvsp[0].Number;
@@ -1495,18 +1493,18 @@ case 39:
 	}
 break;
 case 40:
-#line 313 "getdate.y"
+#line 310 "getdate.y"
 {
 	    yyval.Meridian = MER24;
 	}
 break;
 case 41:
-#line 316 "getdate.y"
+#line 313 "getdate.y"
 {
 	    yyval.Meridian = yyvsp[0].Meridian;
 	}
 break;
-#line 1457 "getdate.c"
+#line 1455 "getdate.c"
     }
     yyssp -= yym;
     yystate = *yyssp;
