@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1996, 1998, 1999 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 1996, 1998, 1999 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -14,15 +14,15 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#ifndef _SUDOERS_INS_CLASSIC_H
-#define _SUDOERS_INS_CLASSIC_H
+#ifndef SUDOERS_INS_CLASSIC_H
+#define SUDOERS_INS_CLASSIC_H
 
     /*
      * Insults from the original sudo(8).
      */
 
     "Wrong!  You cheating scum!",
-#ifdef PC_INSULTS
+#ifndef OFFENSIVE_INSULTS
     "And you call yourself a Rocket Scientist!",
 #else
     "No soap, honkie-lips.",
@@ -34,4 +34,4 @@
     "Do you think like you type?",
     "Your mind just hasn't been the same since the electro-shock, has it?",
 
-#endif /* _SUDOERS_INS_CLASSIC_H */
+#endif /* SUDOERS_INS_CLASSIC_H */
