@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2014 Todd C. Miller <Todd.Miller@courtesan.com>
+ * Copyright (c) 2012-2016 Todd C. Miller <Todd.Miller@sudo.ws>
  *
  * Permission to use, copy, modify, and distribute this software for any
  * purpose with or without fee is hereby granted, provided that the above
@@ -18,14 +18,7 @@
 
 #include <sys/types.h>
 #include <stdio.h>
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif /* STDC_HEADERS */
+#include <stdlib.h>
 #ifdef HAVE_STRING_H
 # include <string.h>
 #endif /* HAVE_STRING_H */
@@ -43,7 +36,8 @@
 
 #include "sudo_compat.h"
 #include "sudo_fatal.h"
-#include "sudo_alloc.h"
+#include "sudoers_debug.h"
+#include "defaults.h"
 #include "logging.h"
 
 static int current_locale = SUDOERS_LOCALE_USER;
@@ -53,20 +47,28 @@ static char *sudoers_locale;
 int
 sudoers_getlocale(void)
 {
-    return current_locale;
+    debug_decl(sudoers_getlocale, SUDOERS_DEBUG_UTIL)
+    debug_return_int(current_locale);
 }
 
-void
+bool
 sudoers_initlocale(const char *ulocale, const char *slocale)
 {
+    debug_decl(sudoers_initlocale, SUDOERS_DEBUG_UTIL)
+
     if (ulocale != NULL) {
-	sudo_efree(user_locale);
-	user_locale = sudo_estrdup(ulocale);
+	free(user_locale);
+	if ((user_locale = strdup(ulocale)) == NULL)
+	    debug_return_bool(false);
     }
     if (slocale != NULL) {
-	sudo_efree(sudoers_locale);
-	sudoers_locale = sudo_estrdup(slocale);
+	free(sudoers_locale);
+	if ((sudoers_locale = strdup(slocale)) == NULL)
+	    debug_return_bool(false);
     }
+    sudo_debug_printf(SUDO_DEBUG_INFO, "%s: user locale %s, sudoers locale %s",
+	__func__, user_locale, sudoers_locale);
+    debug_return_bool(true);
 }
 
 /*
@@ -79,6 +81,7 @@ bool
 sudoers_setlocale(int newlocale, int *prevlocale)
 {
     char *res = NULL;
+    debug_decl(sudoers_setlocale, SUDOERS_DEBUG_UTIL)
 
     switch (newlocale) {
 	case SUDOERS_LOCALE_USER:
@@ -86,9 +89,17 @@ sudoers_setlocale(int newlocale, int *prevlocale)
 		*prevlocale = current_locale;
 	    if (current_locale != SUDOERS_LOCALE_USER) {
 		current_locale = SUDOERS_LOCALE_USER;
+		sudo_debug_printf(SUDO_DEBUG_DEBUG,
+		    "%s: setting locale to %s (user)", __func__,
+		    user_locale ? user_locale : "");
 		res = setlocale(LC_ALL, user_locale ? user_locale : "");
-		if (res != NULL && user_locale == NULL)
-		    user_locale = sudo_estrdup(setlocale(LC_ALL, NULL));
+		if (res != NULL && user_locale == NULL) {
+		    user_locale = setlocale(LC_ALL, NULL);
+		    if (user_locale != NULL)
+			user_locale = strdup(user_locale);
+		    if (user_locale == NULL)
+			res = NULL;
+		}
 	    }
 	    break;
 	case SUDOERS_LOCALE_SUDOERS:
@@ -96,44 +107,45 @@ sudoers_setlocale(int newlocale, int *prevlocale)
 		*prevlocale = current_locale;
 	    if (current_locale != SUDOERS_LOCALE_SUDOERS) {
 		current_locale = SUDOERS_LOCALE_SUDOERS;
+		sudo_debug_printf(SUDO_DEBUG_DEBUG,
+		    "%s: setting locale to %s (sudoers)", __func__,
+		    sudoers_locale ? sudoers_locale : "C");
 		res = setlocale(LC_ALL, sudoers_locale ? sudoers_locale : "C");
 		if (res == NULL && sudoers_locale != NULL) {
 		    if (strcmp(sudoers_locale, "C") != 0) {
-			sudo_efree(sudoers_locale);
-			sudoers_locale = sudo_estrdup("C");
-			res = setlocale(LC_ALL, "C");
+			free(sudoers_locale);
+			sudoers_locale = strdup("C");
+			if (sudoers_locale != NULL)
+			    res = setlocale(LC_ALL, "C");
 		    }
 		}
 	    }
 	    break;
     }
-    return res ? true : false;
+    debug_return_bool(res ? true : false);
 }
 
-#ifdef HAVE_LIBINTL_H
-char *
-sudo_warn_gettext_v1(const char *msgid)
+bool
+sudoers_warn_setlocale(bool restore, int *cookie)
 {
-    int warning_locale;
-    char *msg;
+    debug_decl(sudoers_warn_setlocale, SUDOERS_DEBUG_UTIL)
 
-    sudoers_setlocale(SUDOERS_LOCALE_USER, &warning_locale);
-    msg = gettext(msgid);
-    sudoers_setlocale(warning_locale, NULL);
-
-    return msg;
+    if (restore)
+	debug_return_bool(sudoers_setlocale(*cookie, NULL));
+    debug_return_bool(sudoers_setlocale(SUDOERS_LOCALE_USER, cookie));
 }
-#endif /* HAVE_LIBINTL_H */
 
-char *
-sudo_warn_strerror_v1(int errnum)
+/*
+ * Callback for sudoers_locale sudoers setting.
+ */
+bool
+sudoers_locale_callback(const union sudo_defs_val *sd_un)
 {
-    int warning_locale;
-    char *errmsg;
+    debug_decl(sudoers_locale_callback, SUDOERS_DEBUG_UTIL)
 
-    sudoers_setlocale(SUDOERS_LOCALE_USER, &warning_locale);
-    errmsg = strerror(errnum);
-    sudoers_setlocale(warning_locale, NULL);
-
-    return errmsg;
+    if (sudoers_initlocale(NULL, sd_un->str)) {
+	if (setlocale(LC_ALL, sd_un->str) != NULL)
+	    debug_return_bool(true);
+    }
+    debug_return_bool(false);
 }
